@@ -7,10 +7,12 @@
 #undef HAVE_LIBSM
 #undef HAVE_PROGRAM_INVOCATION_SHORT_NAME
 #undef HAVE_PROGRAM_INVOCATION_NAME
-#undef HAVE_MICO_ORB_RUN
 #undef PACKAGE
 #undef VERSION
 
+#undef HAVE_ESD
+#undef HAVE_LIBESD
+
 #undef HAVE_LIBGTOP
 
 /* Define if LibGTop has support for multiple processors. */
