@@ -286,7 +286,7 @@ apply_properties (GtkWidget *widget, gint button_num, gpointer data)
 	Printer *pr = data;
 	apply_one (pr->prop_name, &pr->print_title);
 	apply_one (pr->prop_command, &pr->print_command);
-	gtk_label_set (GTK_LABEL (pr->label), pr->print_title);
+	gtk_label_set_text (GTK_LABEL (pr->label), pr->print_title);
 	position_label (pr->label, pr->ev);
 }
 
@@ -329,9 +329,13 @@ build_label_and_entry (Printer *pr, GtkTable *table, int row, char *label,
 static void
 printer_properties (AppletWidget *applet, gpointer data)
 {
+        static GnomeHelpMenuEntry help_entry = { NULL,
+						 "properties-printer" };
 	GtkWidget *table;
 	Printer *pr = data;
 
+	help_entry.name = gnome_app_id;
+
 	if (pr->printer_prop) {
 		gdk_window_raise(pr->printer_prop->window);
 		return;
@@ -364,6 +368,9 @@ printer_properties (AppletWidget *applet, gpointer data)
 			    GTK_SIGNAL_FUNC(close_properties), pr);
 	gtk_signal_connect (GTK_OBJECT (pr->printer_prop), "destroy",
 			    GTK_SIGNAL_FUNC(close_properties), pr);
+	gtk_signal_connect (GTK_OBJECT (pr->printer_prop), "help",
+			    GTK_SIGNAL_FUNC(gnome_help_pbox_display),
+			    &help_entry);
 			    
 	gtk_widget_show (pr->printer_prop);
 }
