@@ -8,7 +8,7 @@ GtkWidget         **blists = NULL;
 GtkWidget         **flists = NULL;
 gint                blists_num = 0;
 GList              *tasks = NULL;
-PanelOrientType     applet_orient = ORIENT_DOWN;
+PanelOrientType     applet_orient = -1;
 gint                current_desk = 0;
 gint                num_desk = 0;
 gchar              *desk_name[32];
@@ -20,20 +20,36 @@ GtkWidget          *main_box = NULL;
 GtkWidget          *task_table = NULL;
 GdkPixmap          *p_1 = NULL, *p_2 = NULL, *p_3 = NULL;
 GdkPixmap          *m_1 = NULL, *m_2 = NULL, *m_3 = NULL;
-gint                pager_rows = 2;
+
+gint                pager_rows = 1;
 gchar               pager_size = 1;
 gchar               tasks_all = 0;
 gint                task_rows_h = 2;
 gint                task_rows_v = 1;
 gint                max_task_width = 400;
+gint                max_task_vwidth = 48;
 gchar               show_tasks = 1;
 gchar               show_pager = 1;
 gchar               show_icons = 1;
 gchar		    show_arrow = 1;
+
+gint                o_pager_rows = 1;
+gchar               o_pager_size = 1;
+gchar               o_tasks_all = 0;
+gint                o_task_rows_h = 2;
+gint                o_task_rows_v = 1;
+gint                o_max_task_width = 400;
+gint                o_max_task_vwidth = 4;
+gchar               o_show_tasks = 1;
+gchar               o_show_pager = 1;
+gchar               o_show_icons = 1;
+gchar		    o_show_arrow = 1;
+
 gint                area_w = 1;
 gint                area_h = 1;
 gint                area_x = 0;
 gint                area_y = 0;
+gchar               tasks_changed = 0;
 
 #define PAGER_W_0 31
 #define PAGER_H_0 22
@@ -48,6 +64,7 @@ cb_applet_orient_change(GtkWidget *w, PanelOrientType o, gpointer data)
   
   if (o == applet_orient) 
     return;
+
   applet_orient = o;
   switch (o) 
     {
@@ -90,8 +107,7 @@ cb_applet_orient_change(GtkWidget *w, PanelOrientType o, gpointer data)
       init_applet_gui_vert();
       break;
     }
-  w = NULL;
-  data = NULL;
+  gtk_widget_show(w);
 }
 
 
@@ -103,7 +119,7 @@ cb_applet_about(AppletWidget * widget, gpointer data)
     {"The Rasterman", NULL};
   
   about = gnome_about_new
-    (_("Desktop Pager Applet"), "0.1", _("Copyright (C)1998 Red Hat Software"),
+    (_("Desktop Pager Applet"), "0.1", _("Copyright (C)1998,1999 Red Hat Software, Free Software Foundation"),
      authors,
      _("Pager for a GNOME compliant Window Manager"),
      NULL);
@@ -121,9 +137,9 @@ cb_check_show_icons(GtkWidget *widget, gpointer data)
   if (prop)
     gnome_property_box_changed(GNOME_PROPERTY_BOX(prop));
   if (GTK_TOGGLE_BUTTON(widget)->active)
-    show_icons = 1;
+    o_show_icons = 1;
   else
-    show_icons = 0;
+    o_show_icons = 0;
 }
 
 void
@@ -135,9 +151,9 @@ cb_check_show_arrow(GtkWidget *widget, gpointer data)
   if (prop)
     gnome_property_box_changed(GNOME_PROPERTY_BOX(prop));
   if (GTK_TOGGLE_BUTTON(widget)->active)
-    show_arrow = 1;
+    o_show_arrow = 1;
   else
-    show_arrow = 0;
+    o_show_arrow = 0;
 }
 
 void 
@@ -149,9 +165,9 @@ cb_check_pager_size(GtkWidget *widget, gpointer data)
   if (prop)
     gnome_property_box_changed(GNOME_PROPERTY_BOX(prop));
   if (GTK_TOGGLE_BUTTON(widget)->active)
-    pager_size = 1;
+    o_pager_size = 1;
   else
-    pager_size = 0;
+    o_pager_size = 0;
 }
 
 void 
@@ -163,9 +179,9 @@ cb_check_all_tasks(GtkWidget *widget, gpointer data)
   if (prop)
     gnome_property_box_changed(GNOME_PROPERTY_BOX(prop));
   if (GTK_TOGGLE_BUTTON(widget)->active)
-    tasks_all = 1;
+    o_tasks_all = 1;
   else
-    tasks_all = 0;
+    o_tasks_all = 0;
 }
 
 void 
@@ -177,9 +193,9 @@ cb_check_show_tasks(GtkWidget *widget, gpointer data)
   if (prop)
     gnome_property_box_changed(GNOME_PROPERTY_BOX(prop));
   if (GTK_TOGGLE_BUTTON(widget)->active)
-    show_tasks = 1;
+    o_show_tasks = 1;
   else
-    show_tasks = 0;
+    o_show_tasks = 0;
 }
 
 void 
@@ -191,9 +207,9 @@ cb_check_show_pager(GtkWidget *widget, gpointer data)
   if (prop)
     gnome_property_box_changed(GNOME_PROPERTY_BOX(prop));
   if (GTK_TOGGLE_BUTTON(widget)->active)
-    show_pager = 1;
+    o_show_pager = 1;
   else
-    show_pager = 0;
+    o_show_pager = 0;
 }
 
 void 
@@ -204,7 +220,18 @@ cb_adj_max_width(GtkAdjustment *adj, GtkAdjustment *adj1)
   prop = gtk_object_get_data(GTK_OBJECT(adj), "prop");
   if (prop)
     gnome_property_box_changed(GNOME_PROPERTY_BOX(prop));
-  max_task_width = (gint)(adj->value);
+  o_max_task_width = (gint)(adj->value);
+}
+
+void 
+cb_adj_max_vwidth(GtkAdjustment *adj, GtkAdjustment *adj1)
+{
+  GtkWidget *prop;
+  
+  prop = gtk_object_get_data(GTK_OBJECT(adj), "prop");
+  if (prop)
+    gnome_property_box_changed(GNOME_PROPERTY_BOX(prop));
+  o_max_task_vwidth = (gint)(adj->value);
 }
 
 void 
@@ -215,7 +242,7 @@ cb_adj_rows_h(GtkAdjustment *adj, GtkAdjustment *adj1)
   prop = gtk_object_get_data(GTK_OBJECT(adj), "prop");
   if (prop)
     gnome_property_box_changed(GNOME_PROPERTY_BOX(prop));
-  task_rows_h = (gint)(adj->value);
+  o_task_rows_h = (gint)(adj->value);
 }
 
 void 
@@ -226,7 +253,7 @@ cb_adj_rows_v(GtkAdjustment *adj, GtkAdjustment *adj1)
   prop = gtk_object_get_data(GTK_OBJECT(adj), "prop");
   if (prop)
     gnome_property_box_changed(GNOME_PROPERTY_BOX(prop));
-  task_rows_v = (gint)(adj->value);
+  o_task_rows_v = (gint)(adj->value);
 }
 
 void 
@@ -237,20 +264,25 @@ cb_adj_rows(GtkAdjustment *adj, GtkAdjustment *adj1)
   prop = gtk_object_get_data(GTK_OBJECT(adj), "prop");
   if (prop)
     gnome_property_box_changed(GNOME_PROPERTY_BOX(prop));
-  pager_rows = (gint)(adj->value);
-}
-
-void
-cb_prop_cancel(GtkWidget *widget, gpointer data)
-{
-  widget = NULL;
-  data = NULL;
+  o_pager_rows = (gint)(adj->value);
 }
 
 void
 cb_prop_apply(GtkWidget *widget, gpointer data)
 {
   gint i;
+
+  pager_rows = o_pager_rows;
+  pager_size = o_pager_size;
+  tasks_all = o_tasks_all;
+  task_rows_h = o_task_rows_h;
+  task_rows_v = o_task_rows_v;
+  max_task_width = o_max_task_width;
+  max_task_vwidth = o_max_task_vwidth;
+  show_tasks = o_show_tasks;
+  show_pager = o_show_pager;
+  show_icons = o_show_icons;
+  show_arrow = o_show_arrow;;
   
   switch (applet_orient) 
     {
@@ -293,19 +325,6 @@ cb_prop_apply(GtkWidget *widget, gpointer data)
       init_applet_gui_vert();
       break;
     }
-  gnome_config_set_int("gnome_pager/stuff/pager_rows", pager_rows);
-  gnome_config_set_int("gnome_pager/stuff/pager_size", pager_size);
-  gnome_config_set_int("gnome_pager/stuff/tasks_all", tasks_all);
-  gnome_config_set_int("gnome_pager/stuff/task_rows_h", task_rows_h);
-  gnome_config_set_int("gnome_pager/stuff/task_rows_v", task_rows_v);
-  gnome_config_set_int("gnome_pager/stuff/max_task_width", max_task_width);
-  gnome_config_set_int("gnome_pager/stuff/show_tasks", show_tasks);
-  gnome_config_set_int("gnome_pager/stuff/show_pager", show_pager);
-  gnome_config_set_int("gnome_pager/stuff/show_icons", show_icons);
-  gnome_config_set_int("gnome_pager/stuff/show_arrow", show_arrow);
-  gnome_config_sync();
-  widget = NULL;
-  data = NULL;
 }
 
 void 
@@ -317,12 +336,24 @@ cb_applet_properties(AppletWidget * widget, gpointer data)
   GtkAdjustment *adj;
 
   help_entry.name = gnome_app_id;
-
+  
+  o_pager_rows = pager_rows;
+  o_pager_size = pager_size;
+  o_tasks_all = tasks_all;
+  o_task_rows_h = task_rows_h;
+  o_task_rows_v = task_rows_v;
+  o_max_task_width = max_task_width;
+  o_max_task_vwidth = max_task_vwidth;
+  o_show_tasks = show_tasks;
+  o_show_pager = show_pager;
+  o_show_icons = show_icons;
+  o_show_arrow = show_arrow;
+  
   if (!prop) 
     {
       prop = gnome_property_box_new();
       gtk_signal_connect (GTK_OBJECT(prop), "delete_event",
-			  GTK_SIGNAL_FUNC(cb_prop_cancel), NULL);
+			  GTK_SIGNAL_FUNC(gtk_false),NULL);
       gtk_signal_connect (GTK_OBJECT(prop), "apply",
 			  GTK_SIGNAL_FUNC(cb_prop_apply), NULL);
       gtk_signal_connect (GTK_OBJECT(prop), "help",
@@ -397,6 +428,22 @@ cb_applet_properties(AppletWidget * widget, gpointer data)
 		       0, 1, 0, 1, GTK_FILL|GTK_EXPAND,0,0,0);
       gtk_table_attach(GTK_TABLE(table), spin, 
 		       1, 2, 0, 1, GTK_FILL|GTK_EXPAND,0,0,0);
+      
+      adj = (GtkAdjustment *)gtk_adjustment_new((gfloat)max_task_vwidth, 4, 
+						(gfloat)gdk_screen_width(), 
+						4, 4, 4 );
+      gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
+			 GTK_SIGNAL_FUNC(cb_adj_max_vwidth), adj);
+      gtk_object_set_data(GTK_OBJECT(adj), "prop", prop);
+      label = gtk_label_new(_("Maximum width of vertical task list"));
+      gtk_widget_show(label);
+      spin = gtk_spin_button_new(adj, 1, 0);
+      gtk_widget_show(spin);
+      gtk_table_attach(GTK_TABLE(table), label, 
+		       0, 1, 1, 2, GTK_FILL|GTK_EXPAND,0,0,0);
+      gtk_table_attach(GTK_TABLE(table), spin, 
+		       1, 2, 1, 2, GTK_FILL|GTK_EXPAND,0,0,0);
+
       adj = (GtkAdjustment *)gtk_adjustment_new((gfloat)task_rows_h, 1, 
 						8, 1, 1, 1 );
       gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
@@ -407,9 +454,9 @@ cb_applet_properties(AppletWidget * widget, gpointer data)
       spin = gtk_spin_button_new(adj, 1, 0);
       gtk_widget_show(spin);
       gtk_table_attach(GTK_TABLE(table), label, 
-		       0, 1, 1, 2, GTK_FILL|GTK_EXPAND,0,0,0);
+		       0, 1, 2, 3, GTK_FILL|GTK_EXPAND,0,0,0);
       gtk_table_attach(GTK_TABLE(table), spin, 
-		       1, 2, 1, 2, GTK_FILL|GTK_EXPAND,0,0,0);
+		       1, 2, 2, 3, GTK_FILL|GTK_EXPAND,0,0,0);
       adj = (GtkAdjustment *)gtk_adjustment_new((gfloat)task_rows_v, 1, 
 						4, 1, 1, 1 );
       gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
@@ -420,9 +467,9 @@ cb_applet_properties(AppletWidget * widget, gpointer data)
       spin = gtk_spin_button_new(adj, 1, 0);
       gtk_widget_show(spin);
       gtk_table_attach(GTK_TABLE(table), label, 
-		       0, 1, 2, 3, GTK_FILL|GTK_EXPAND,0,0,0);
+		       0, 1, 3, 4, GTK_FILL|GTK_EXPAND,0,0,0);
       gtk_table_attach(GTK_TABLE(table), spin, 
-		       1, 2, 2, 3, GTK_FILL|GTK_EXPAND,0,0,0);
+		       1, 2, 3, 4, GTK_FILL|GTK_EXPAND,0,0,0);
       adj = (GtkAdjustment *)gtk_adjustment_new((gfloat)pager_rows, 1, 
 						8, 1, 1, 1 );
       gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
@@ -433,15 +480,13 @@ cb_applet_properties(AppletWidget * widget, gpointer data)
       spin = gtk_spin_button_new(adj, 1, 0);
       gtk_widget_show(spin);
       gtk_table_attach(GTK_TABLE(table), label, 
-		       0, 1, 3, 4, GTK_FILL|GTK_EXPAND,0,0,0);
+		       0, 1, 4, 5, GTK_FILL|GTK_EXPAND,0,0,0);
       gtk_table_attach(GTK_TABLE(table), spin, 
-		       1, 2, 3, 4, GTK_FILL|GTK_EXPAND,0,0,0);
+		       1, 2, 4, 5, GTK_FILL|GTK_EXPAND,0,0,0);
       
     }
   if (prop)
     gtk_widget_show(prop);
-  data = NULL;
-  widget = NULL;
 }
 
 
@@ -602,14 +647,19 @@ client_win_show(Task *t)
       ev.data.l[1] = CurrentTime;
       XSendEvent(GDK_DISPLAY(), t->win, False, 0, (XEvent *) & ev);
     }
-  XSetInputFocus(GDK_DISPLAY(), t->win, RevertToPointerRoot, CurrentTime);  
+  if (t->iconified)
+    XMapWindow(GDK_DISPLAY(), t->win);
+  if (!t->focused)
+    XSetInputFocus(GDK_DISPLAY(), t->win, RevertToPointerRoot, CurrentTime);
   XRaiseWindow(GDK_DISPLAY(), t->win);
-  XMapWindow(GDK_DISPLAY(), t->win);
-  if ((!t->sticky) && (t->desktop != current_desk - 1))
+  if ((!t->sticky) && (t->desktop != current_desk - 1) && (!t->iconified))
     gnome_win_hints_set_current_workspace(t->desktop);
-  x = ((gdk_screen_width() * area_x) + (t->x + (t->w / 2)));
-  y = ((gdk_screen_height() * area_y) + (t->y + (t->h / 2)));
-  desktop_set_area(x / gdk_screen_width(), y / gdk_screen_height());
+  if (!t->iconified)
+    {
+      x = ((gdk_screen_width() * area_x) + (t->x + (t->w / 2)));
+      y = ((gdk_screen_height() * area_y) + (t->y + (t->h / 2)));
+      desktop_set_area(x / gdk_screen_width(), y / gdk_screen_height());
+    }
   /***/
   XSync(GDK_DISPLAY(), False);
 }
@@ -769,10 +819,13 @@ custom_popbox_show(GtkWidget * widget)
   widget = NULL;
 }
 
-void
+int
 showpop_cb(GtkWidget *widget, gpointer data)
 {
   custom_popbox_show(popbox);
+  /*XXX: return false even though clicked is a void signal, but all the _event's
+    need a return value*/
+  return FALSE;
 }
 
 /* PROPERTY CHANGE info gathering callbacks */
@@ -781,7 +834,7 @@ cb_task_change(GtkWidget *widget, GdkEventProperty * ev, Task *t)
 {
   gint i, tdesk;
   gchar tsticky;
-  
+
   tsticky = t->sticky;
   tdesk = t->desktop;
   task_get_info(t);
@@ -802,6 +855,8 @@ cb_task_change(GtkWidget *widget, GdkEventProperty * ev, Task *t)
 	{
 	  desktop_draw(t->desktop);
 	  desktop_draw(tdesk);
+	  if (!tasks_all)
+	    populate_tasks();	  
 	}
       else
 	desktop_draw(t->desktop);
@@ -836,8 +891,11 @@ cb_root_prop_change(GtkWidget * widget, GdkEventProperty * ev)
   at = gdk_atom_intern(XA_WIN_CLIENT_LIST, FALSE);
   if ((ev->atom == at) && (ev->state == GDK_PROPERTY_NEW_VALUE))
     {
+      tasks_changed = 0;
       tasks_update();
-      populate_tasks();
+      if (tasks_changed)
+	populate_tasks();
+      tasks_changed = 0;
       return;
     }
 
@@ -924,6 +982,8 @@ cb_root_prop_change(GtkWidget * widget, GdkEventProperty * ev)
 	    }
 	  g_free(da);
 	}
+      if (!tasks_all)
+	populate_tasks();
       desktop_draw(current_desk);
     }
   at = gdk_atom_intern(XA_WIN_WORKSPACE_NAMES, FALSE);
@@ -969,7 +1029,8 @@ cb_filter_intercept(GdkXEvent *gdk_xevent, GdkEvent *event, gpointer data)
       while (ptr)
 	{
 	  t = (Task *)(ptr->data);
-	  if (t->win == xevent->xconfigure.window)
+	  if (t->win == xevent->xconfigure.window ||
+	      t->frame == xevent->xconfigure.window)
 	    {
 	      cb_task_change(NULL, (GdkEventProperty *)event, t);
 	      return GDK_FILTER_CONTINUE;
@@ -1002,7 +1063,7 @@ void
 task_get_info(Task *t)
 {
   GnomeWinState       *win_state;
-  Window               w2, ret, root, *wl;
+  Window               ret;
   gchar               *name;
   gint                 size;
   guint                w, h, d;
@@ -1010,6 +1071,7 @@ task_get_info(Task *t)
   Atom                 a;
   int                  rev;
   gchar               psticky;
+  gchar               needredo = 0;
 
   gdk_error_warnings = 0;
   /* is this window focused */
@@ -1070,35 +1132,68 @@ task_get_info(Task *t)
     }
   
   /* geometry!!!!!!!!!!! */
-  XGetGeometry(GDK_DISPLAY(), t->win, &ret, &size, &size, &w, &h, &d, &d);
+  get_window_root_and_frame_id(t->win, &t->frame, &t->root);
+
+  XGetGeometry(GDK_DISPLAY(), t->frame, &ret, &size, &size, &w, &h, &d, &d);
   t->w = (gint)w;
   t->h = (gint)h;
-  wl = NULL;
-  w2 = t->win;
-  while (XQueryTree(GDK_DISPLAY(), w2, &root, &ret, &wl, &size))
-    {
-      if ((wl) && (size > 0))
-	XFree(wl);
-      w2 = ret;
-      if (ret == root)
-	break;
-      val = util_get_atom(ret, "ENLIGHTENMENT_DESKTOP", XA_CARDINAL, &size);
-      if (val)
-	{
-	  break;
-	  g_free(val);
-	}
-    }
-  /* Wheee w2 is our actualy "root" window even on virtual root windows */
-  XTranslateCoordinates(GDK_DISPLAY(), t->win, w2, 0, 0, 
+  XTranslateCoordinates(GDK_DISPLAY(), t->frame, t->root, 0, 0, 
 			&(t->x), &(t->y), &ret);
-  if (psticky != t->sticky)
+  /* what area is it on ? */
+  val = util_get_atom(t->win, "_WIN_AREA", XA_CARDINAL, &size);
+  if (val)
+    {
+      gint pax, pay;
+      
+      pax = t->ax;
+      pay = t->ay;
+      t->ax = val[0];
+      t->ay = val[1];
+      g_free(val);
+      
+      if ((pax != t->ax) || (pay != t->ay))
+	needredo = 1;
+    }
+  else
+    {
+      gint sw, sh;
+      gint pax, pay;
+      
+      pax = t->ax;
+      pay = t->ay;
+      sw = gdk_screen_width();
+      sh = gdk_screen_width();
+      if (t->x > sw)
+	t->ax = area_x + (t->x / sw);
+      else if ((t->x + t->w) <= 0)
+	t->ax = area_x + (t->x / sw) - 1;
+      else
+	t->ax = area_x;
+      if (t->y > sh)
+	t->ay = area_y + (t->y / sh);
+      else if ((t->y + t->h) <= 0)
+	t->ay = area_y + (t->y / sh) - 1;
+      else
+	t->ay = area_y;
+      if (t->ax < 0)
+	t->ax = 0;
+      if (t->ay < 0)
+	t->ay = 0;
+      if (t->ax >= area_w)
+	t->ax = area_w - 1;
+      if (t->ay >= area_h)
+	t->ay = area_h - 1;
+      
+      if ((pax != t->ax) || (pay != t->ay))
+	needredo = 1;
+    }
+  if ((psticky != t->sticky) || (needredo))
     {
       populate_tasks();
     }
 }
 
-void 
+gint
 task_add(Window win)
 {
   Task               *t;
@@ -1123,7 +1218,7 @@ task_add(Window win)
       if ((*win_hints) & WIN_HINTS_SKIP_TASKBAR)
 	{
 	  g_free(win_hints);
-	  return;
+	  return 0;
 	}
       g_free(win_hints);
     }
@@ -1131,12 +1226,17 @@ task_add(Window win)
   /* create task struct */
   t = g_malloc(sizeof(Task));
 
+  /* add this client to the list of clients */
+  tasks = g_list_append(tasks, t);
+
   /* initialize members */
   t->win = win;
   t->x = 0;
   t->y = 0;
   t->w = -1;
   t->h = -1;
+  t->ax = 0;
+  t->ay = 0;
   t->name = NULL;
   t->iconified = 0;
   t->shaded = 0;
@@ -1145,7 +1245,10 @@ task_add(Window win)
   t->desktop = 0;
   t->dummy = NULL;
   t->gdkwin = NULL;
+  t->frame_gdkwin = NULL;
   
+  get_window_root_and_frame_id(t->win, &t->frame, &t->root);
+
   /* create dummy GTK widget to get events from */
   t->gdkwin = gdk_window_lookup(win);
   if (t->gdkwin)
@@ -1158,59 +1261,63 @@ task_add(Window win)
   gdk_window_add_filter(t->gdkwin, cb_filter_intercept, t->dummy);  
   /* fake events form win producing signals on dummy widget */
 /*  gdk_window_set_user_data(t->gdkwin, t->dummy);*/
-  /* conntect to "faked" signals */
-/*  gtk_signal_connect(GTK_OBJECT(t->dummy), "property_notify_event",
-		     GTK_SIGNAL_FUNC(cb_task_change), t);
-  gtk_signal_connect(GTK_OBJECT(t->dummy), "focus_in_event",
-		     GTK_SIGNAL_FUNC(cb_task_change), t);
-  gtk_signal_connect(GTK_OBJECT(t->dummy), "focus_out_event",
-		     GTK_SIGNAL_FUNC(cb_task_change), t);
-  gtk_signal_connect(GTK_OBJECT(t->dummy), "configure_event",
-		     GTK_SIGNAL_FUNC(cb_task_change), t);
-*/
+
+  /* DO it all again for the frame window */
+  t->frame_gdkwin = gdk_window_lookup(t->frame);
+  if (t->frame_gdkwin)
+    gdk_window_ref(t->frame_gdkwin);
+  else
+    t->frame_gdkwin = gdk_window_foreign_new(t->frame);
+  gdk_window_add_filter(t->frame_gdkwin, cb_filter_intercept, t->dummy);  
+
   /* make sure we get the events */
   XSelectInput(GDK_DISPLAY(), win, PropertyChangeMask | FocusChangeMask |
 	       StructureNotifyMask);
-  /* add this client to the list of clients */
-  tasks = g_list_append(tasks, t);
+
+  /* make sure we get the events */
+  XSelectInput(GDK_DISPLAY(), t->frame, StructureNotifyMask);
 
   /* get info about task */
   task_get_info(t);
+  return 1;
 }
 
 
 void 
 task_delete(Window win)
 {
-  guint               i, n, tdesk;
+  guint               i, tdesk;
   Task               *t;
   gchar               tstick;
+  GList              *ptr;
 
   gdk_error_warnings = 0;
-  n = g_list_length(tasks);
-  for (i = 0; i < n; i++)
+  ptr = tasks;
+  while(ptr)
     {
-      t = (g_list_nth(tasks, i))->data;
-      if (win == t->win)
+      t = ptr->data;
+      ptr = ptr->next;
+      if (t)
 	{
-	  tstick = t->sticky;
-	  tdesk = t->desktop;
-	  if (t->name)
-	    g_free(t->name);
-	  if (t->gdkwin)
-	    gdk_window_unref(t->gdkwin);
-/*	  if (t->dummy)
-	    gtk_widget_destroy(t->dummy);*/
-	  tasks = g_list_remove(tasks, t);
-	  g_free(t);
-	  if (!tstick)
-	    desktop_draw(tdesk);
-	  else
+	  if (win == t->win)
 	    {
-	      for (i = 0; i < (guint)num_desk; i++)
-		desktop_draw(i);
+	      tasks = g_list_remove(tasks, t);
+	      tstick = t->sticky;
+	      tdesk = t->desktop;
+	      if (t->name)
+		g_free(t->name);
+	      if (t->gdkwin)
+		gdk_window_unref(t->gdkwin);
+	      g_free(t);
+	      if (!tstick)
+		desktop_draw(tdesk);
+	      else
+		{
+		  for (i = 0; i < (guint)num_desk; i++)
+		    desktop_draw(i);
+		}
+	      return;
 	    }
-	  return;
 	}
     }
 }
@@ -1255,7 +1362,10 @@ tasks_match(Window * win, guint num)
 		}
 	    }
 	  if (!there)
-	    task_delete(t->win);
+	    {
+	      tasks_changed = 1;
+	      task_delete(t->win);
+	    }
 	}
     }
 
@@ -1274,7 +1384,10 @@ tasks_match(Window * win, guint num)
 	    }
 	}
       if (!there)
-	task_add(win[i]);
+	{
+	  if (task_add(win[i]))
+	    tasks_changed = 1;
+	}
     }
 }
 
@@ -1325,6 +1438,32 @@ get_desktop_names(void)
     g_list_free(gl);
 }
 
+/* sesion save signal handler*/
+static gint
+cb_applet_save_session(GtkWidget *w,
+		       const char *privcfgpath,
+		       const char *globcfgpath)
+{
+  /*XXX: if any of this applies to pager stuff in general, push
+    globcfgpath, and use a descriptive section name such as gnome_pager*/
+  gnome_config_push_prefix(privcfgpath);
+  gnome_config_set_int("stuff/pager_rows", pager_rows);
+  gnome_config_set_int("stuff/pager_size", pager_size);
+  gnome_config_set_int("stuff/tasks_all", tasks_all);
+  gnome_config_set_int("stuff/task_rows_h", task_rows_h);
+  gnome_config_set_int("stuff/task_rows_v", task_rows_v);
+  gnome_config_set_int("stuff/max_task_width", max_task_width);
+  gnome_config_set_int("stuff/max_task_vwidth", max_task_vwidth);
+  gnome_config_set_int("stuff/show_tasks", show_tasks);
+  gnome_config_set_int("stuff/show_pager", show_pager);
+  gnome_config_set_int("stuff/show_icons", show_icons);
+  gnome_config_set_int("stuff/show_arrow", show_arrow);
+  gnome_config_pop_prefix();
+  gnome_config_sync();
+  gnome_config_drop_all();
+  return FALSE;
+}
+
 /* select on events for the root window properies */
 void
 select_root_properties(void)
@@ -1387,16 +1526,65 @@ main(int argc, char *argv[])
       exit(1);
     }
   
-  pager_rows = gnome_config_get_int("gnome_pager/stuff/pager_rows=2");
-  pager_size = gnome_config_get_int("gnome_pager/stuff/pager_size=1");
-  tasks_all = gnome_config_get_int("gnome_pager/stuff/tasks_all=0");
-  task_rows_h = gnome_config_get_int("gnome_pager/stuff/task_rows_h=2");
-  task_rows_v = gnome_config_get_int("gnome_pager/stuff/task_rows_v=1");
-  max_task_width = gnome_config_get_int("gnome_pager/stuff/max_task_width=400");
-  show_tasks = gnome_config_get_int("gnome_pager/stuff/show_tasks=1");
-  show_pager = gnome_config_get_int("gnome_pager/stuff/show_pager=1");
-  show_icons = gnome_config_get_int("gnome_pager/stuff/show_icons=1");
-  show_arrow = gnome_config_get_int("gnome_pager/stuff/show_arrow=1");
+  /*FIXME: remove this later!!!!!, in favour of the WELL BEHAVED
+    session saving/loading, leave it in for now so that people that
+    have some saved config get it*/
+  /**/pager_rows = gnome_config_get_int("gnome_pager/stuff/pager_rows=2");
+  /**/pager_size = gnome_config_get_int("gnome_pager/stuff/pager_size=1");
+  /**/tasks_all = gnome_config_get_int("gnome_pager/stuff/tasks_all=0");
+  /**/task_rows_h = gnome_config_get_int("gnome_pager/stuff/task_rows_h=2");
+  /**/task_rows_v = gnome_config_get_int("gnome_pager/stuff/task_rows_v=1");
+  /**/max_task_width = gnome_config_get_int("gnome_pager/stuff/max_task_width=400");
+  /**/max_task_vwidth = gnome_config_get_int("gnome_pager/stuff/max_task_vwidth=48");
+  /**/show_tasks = gnome_config_get_int("gnome_pager/stuff/show_tasks=1");
+  /**/show_pager = gnome_config_get_int("gnome_pager/stuff/show_pager=1");
+  /**/show_icons = gnome_config_get_int("gnome_pager/stuff/show_icons=1");
+  /**/show_arrow = gnome_config_get_int("gnome_pager/stuff/show_arrow=1");
+
+  /*need to create the applet widget before we can get config data, so
+    that we know where to get them from*/
+  applet = applet_widget_new("gnomepager_applet");
+  if (!applet)
+    {
+      g_error("Can't create applet!\n");
+      exit(1);
+    }
+  gtk_signal_connect(GTK_OBJECT(applet), "change_orient",
+		     GTK_SIGNAL_FUNC(cb_applet_orient_change),
+		     NULL);
+  gtk_signal_connect(GTK_OBJECT(applet), "save_session",
+		     GTK_SIGNAL_FUNC(cb_applet_save_session),
+		     NULL);
+
+  init_applet_gui();
+
+  applet_widget_register_stock_callback(APPLET_WIDGET(applet),
+					"about",
+					GNOME_STOCK_MENU_ABOUT,
+					_("About..."),
+					cb_applet_about,
+					NULL);
+  applet_widget_register_stock_callback(APPLET_WIDGET(applet),
+					"properties",
+					GNOME_STOCK_MENU_PROP,
+					_("Properties..."),
+					cb_applet_properties,
+					NULL);
+
+  /*this really loads the correct data*/
+  gnome_config_push_prefix(APPLET_WIDGET(applet)->privcfgpath);
+  pager_rows = gnome_config_get_int("stuff/pager_rows=2");
+  pager_size = gnome_config_get_int("stuff/pager_size=1");
+  tasks_all = gnome_config_get_int("stuff/tasks_all=0");
+  task_rows_h = gnome_config_get_int("stuff/task_rows_h=2");
+  task_rows_v = gnome_config_get_int("stuff/task_rows_v=1");
+  max_task_width = gnome_config_get_int("stuff/max_task_width=400");
+  max_task_vwidth = gnome_config_get_int("stuff/max_task_vwidth=48");
+  show_tasks = gnome_config_get_int("stuff/show_tasks=1");
+  show_pager = gnome_config_get_int("stuff/show_pager=1");
+  show_icons = gnome_config_get_int("stuff/show_icons=1");
+  show_arrow = gnome_config_get_int("stuff/show_arrow=1");
+  gnome_config_pop_prefix();
 
   gdk_error_warnings = 0;  
   get_desktop_names();
@@ -1438,31 +1626,7 @@ main(int argc, char *argv[])
   gdk_imlib_data_to_pixmap(icon2_xpm, &p_2, &m_2);
   gdk_imlib_data_to_pixmap(icon3_xpm, &p_3, &m_3);
   printf("%p %p %p\n", p_1, p_2, p_3);
-  applet = applet_widget_new("gnomepager_applet");
-  if (!applet)
-    {
-      g_error("Can't create applet!\n");
-      exit(1);
-    }
-  gtk_signal_connect(GTK_OBJECT(applet), "change_orient",
-		     GTK_SIGNAL_FUNC(cb_applet_orient_change),
-		     NULL);
-  applet_widget_register_stock_callback(APPLET_WIDGET(applet),
-					"about",
-					GNOME_STOCK_MENU_ABOUT,
-					_("About..."),
-					cb_applet_about,
-					NULL);
-  applet_widget_register_stock_callback(APPLET_WIDGET(applet),
-					"properties",
-					GNOME_STOCK_MENU_PROP,
-					_("Properties..."),
-					cb_applet_properties,
-					NULL);
-  gtk_widget_realize(applet);
-  init_applet_gui();
-  gtk_widget_show(applet);
-  
+
   applet_widget_gtk_main();
   return 0;
 }
@@ -1470,13 +1634,11 @@ main(int argc, char *argv[])
 void
 init_applet_gui(void)
 {
-  gtk_window_set_policy(GTK_WINDOW(applet), 1, 1, 1);
-
   if (!hold_box)
     {
       hold_box = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
-      gtk_widget_show(hold_box);
       applet_widget_add(APPLET_WIDGET(applet), hold_box);
+      gtk_widget_show(hold_box);
     }
 }
 
@@ -1550,29 +1712,137 @@ desktop_set_area(int ax, int ay)
 	               SubstructureNotifyMask, (XEvent*) &xev);
 }
 
-#include "stripe.xbm"
 
 void
-desktop_cb_redraw(GtkWidget *widget, gpointer data)
+get_window_root_and_frame_id(Window w, Window *ret_frame, Window *ret_root)
 {
+  Window               w3, w2, ret, root, *wl;
+  gint                 size;
+  CARD32              *val;
+
+  w3 = w2 = w;
+
+  while (XQueryTree(GDK_DISPLAY(), w2, &root, &ret, &wl, &size))
+    {
+      if ((wl) && (size > 0))
+	XFree(wl);
+      w3 = w2;
+      w2 = ret;
+      if (ret == root)
+	break;
+      val = util_get_atom(ret, "ENLIGHTENMENT_DESKTOP", XA_CARDINAL, &size);
+      if (val)
+	{
+	  break;
+	  g_free(val);
+	}
+    }
+
+  *ret_root = w2;
+  *ret_frame = w3;
+
+  return;
+}
+
+
+
+GList  *get_task_stacking(GList *tasks, gint desk)
+{
+  GList *tasks_on_desk=NULL;
+  GList *root_ids=NULL;
+  GList *retval=NULL;
+  GList *p, *p2, *p3;
+  Task *t;
+  Window dummy1, dummy2, root, *wl;
+  gint size, count;
+
+  /* Find the tasks that are sticky or on the specified desk 
+     which are not iconified. */
+
+  for (p = tasks; p != NULL; p = p->next) {
+    t = (Task *)p->data;
+    
+    if (((t->desktop == desk) || (t->sticky)) && (!(t->iconified))) {
+      tasks_on_desk = g_list_prepend(tasks_on_desk, (gpointer) t);
+    }
+  }
+  tasks_on_desk = g_list_reverse(tasks_on_desk);
+
+  /* Find the IDs of all root or virtual root windows they 
+     may be on. */
+
+  for (p = tasks_on_desk; p != NULL; p = p->next) {
+    t = (Task *)p->data;
+
+    if (NULL == g_list_find(root_ids, (gpointer) t->root)) {
+      root_ids = g_list_prepend(root_ids, (gpointer) t->root);
+    }
+  }
+
+  /* XQueryTree each virtual root window, and for each window ID we
+     get back that matches a frame window, add the corresponding task
+     structure to retval. If we have tasks claiming to be on the same
+     desk which are on different "root" windows, the stacking order
+     between windows on the different roots will be aribtrary, but
+     that is a WM bug (or temporary state) anyway. */
+  
+  for (p = root_ids; p != NULL; p = p->next) {
+    root = (Window) p->data;
+    
+    wl = NULL;
+    size = 0;
+    if (XQueryTree(GDK_DISPLAY(), root, &dummy1, &dummy2, &wl, &size))
+      {
+	for (count = 0; count < size; count++) {
+	  Window w = wl[count];
+	  
+	  for (p2 = tasks_on_desk; p2 != NULL; p2 = p2->next) {
+	    t = (Task *)p2->data;
+	    
+	    if (t->frame == wl[count]) {
+	      retval = g_list_prepend(retval, (gpointer) t);
+	      tasks_on_desk=g_list_remove_link(tasks_on_desk, p2);
+	      break;
+	    }
+	  }
+	}
+	if ((count > 0) && (wl))
+	  XFree(wl);
+      }
+  }
+  retval = g_list_reverse(retval);
+
+  g_list_free(tasks_on_desk);
+  g_list_free(root_ids);
+
+  return retval;
+}
+
+
+#include "stripe.xbm"
+
+int
+actual_redraw(gpointer data)
+{
+  GtkWidget *widget = data;
+  guint current_timeout;
   GtkStyle *s;
   GdkWindow *win;
   gint sel, sw, sh, w, h, desk, x, y, ww, hh;
   Task *t;
-  GList *p;
+  GList *p, *stacking;
   GdkBitmap *bm;
   GdkGC *gc;
 
   if (!widget)
-    return;
+    return TRUE;
   if (!widget->window)
-    return;
+    return TRUE;
   if (!widget->style)
-    return;
+    return TRUE;
   if (!(show_pager))
-    return;
+    return TRUE;
   desk = GPOINTER_TO_INT (gtk_object_get_data(GTK_OBJECT(widget), "desktop"));
-
   /* FIXME: sel is currently always zero */
   sel = GPOINTER_TO_INT (gtk_object_get_data(GTK_OBJECT(widget), "select"));
 
@@ -1587,15 +1857,15 @@ desktop_cb_redraw(GtkWidget *widget, gpointer data)
   if (desk == current_desk)
     {
       gc = s->bg_gc[GTK_STATE_SELECTED];
-      gtk_draw_box(s, win, GTK_STATE_ACTIVE, GTK_SHADOW_IN, 0, 0, -1, -1);
+      gtk_draw_box(s, win,GTK_STATE_ACTIVE, GTK_SHADOW_IN, 0, 0, -1, -1);
     }
   else
     {
       gc = s->base_gc[GTK_STATE_SELECTED];
-      gtk_draw_box(s, win, GTK_STATE_ACTIVE, GTK_SHADOW_OUT, 0, 0, -1, -1);
+      gtk_draw_box(s, win,GTK_STATE_ACTIVE, GTK_SHADOW_OUT, 0, 0, -1, -1);
     }
   
-  bm = gdk_bitmap_create_from_data(win, stripe_bits, stripe_width, stripe_height);
+  bm = gdk_bitmap_create_from_data(win,stripe_bits, stripe_width, stripe_height);
   gdk_gc_set_clip_mask(gc, NULL);
   gdk_gc_set_fill(gc, GDK_STIPPLED);
   gdk_gc_set_stipple(gc, bm);
@@ -1603,7 +1873,7 @@ desktop_cb_redraw(GtkWidget *widget, gpointer data)
 
   x = 2 + ((area_x * w) / area_w);
   y = 2 + ((area_y * h) / area_h);
-  gdk_draw_rectangle(win, gc, TRUE, 
+  gdk_draw_rectangle(win,gc, TRUE, 
 		     x, y, w / area_w, h / area_h); 
 
   gdk_gc_set_stipple(gc, NULL);
@@ -1611,10 +1881,11 @@ desktop_cb_redraw(GtkWidget *widget, gpointer data)
   gdk_pixmap_unref(bm);
   
   for (x = 1; x < area_w; x++)
-    gtk_draw_vline(s, win, GTK_STATE_NORMAL, 2, 1 + h, 1 + ((x * w) / area_w));
+    gtk_draw_vline(s, win,GTK_STATE_NORMAL, 2, 1 + h, 1 + ((x * w) / area_w));
   for (y = 1; y < area_h; y++)
-    gtk_draw_hline(s, win, GTK_STATE_NORMAL, 2, 1 + w, 1 + ((y * h) / area_h));
-  p = tasks;
+    gtk_draw_hline(s, win,GTK_STATE_NORMAL, 2, 1 + w, 1 + ((y * h) / area_h));
+  stacking = get_task_stacking(tasks, desk);
+  p = stacking;
   while (p)
     {
       t = (Task *)p->data;
@@ -1625,15 +1896,49 @@ desktop_cb_redraw(GtkWidget *widget, gpointer data)
 	  y = 2 +((t->y + (area_y * gdk_screen_height()))* h) / (sh * area_h);
 	  hh = (t->h * h) / (sh * area_h);
 	  if (t->focused)
-	    gtk_draw_box(s, win, GTK_STATE_PRELIGHT, GTK_SHADOW_OUT, 
+	    gtk_draw_box(s, win,GTK_STATE_PRELIGHT, GTK_SHADOW_OUT, 
 			 x, y, ww, hh);
 	  else
-	    gtk_draw_box(s, win, GTK_STATE_NORMAL, GTK_SHADOW_ETCHED_IN, 
+	    gtk_draw_box(s, win,GTK_STATE_NORMAL, GTK_SHADOW_ETCHED_IN, 
 			 x, y, ww, hh);
 	    
 	}
       p = p->next;
     }
+  g_list_free(stacking);
+  
+  current_timeout =
+    GPOINTER_TO_INT (gtk_object_get_data(GTK_OBJECT(widget), "timeout"));
+  gtk_object_set_data(GTK_OBJECT(widget), "timeout", 0);
+  return FALSE;
+}
+
+int
+desktop_cb_redraw(GtkWidget *widget, gpointer data)
+{
+  guint current_timeout;
+  
+  current_timeout = 
+    GPOINTER_TO_INT (gtk_object_get_data(GTK_OBJECT(widget), "timeout"));
+  if (current_timeout)
+    gtk_timeout_remove(current_timeout);
+  current_timeout = gtk_timeout_add(200, 
+				    actual_redraw, 
+				    widget);
+  gtk_object_set_data(GTK_OBJECT(widget), "timeout", GINT_TO_POINTER(current_timeout));
+  data = NULL;
+  return FALSE;
+}
+
+void
+cb_desk_destroy(GtkWidget *widget, gpointer data)
+{
+  gint current_timeout;
+  
+  current_timeout =
+    GPOINTER_TO_INT (gtk_object_get_data(GTK_OBJECT(widget), "timeout"));
+  if (current_timeout)
+    gtk_timeout_remove(current_timeout);
 }
 
 GtkWidget *
@@ -1660,6 +1965,8 @@ make_desktop_pane(gint desktop, gint width, gint height)
 		     GTK_SIGNAL_FUNC(desktop_cb_button_up), NULL);
   gtk_signal_connect(GTK_OBJECT(area), "expose_event",
 		     GTK_SIGNAL_FUNC(desktop_cb_redraw), NULL);
+  gtk_signal_connect(GTK_OBJECT(area), "destroy",
+		     GTK_SIGNAL_FUNC(cb_desk_destroy), NULL);
   s = desk_name[desktop];
   if (s) 
     {
@@ -2128,7 +2435,12 @@ set_task_info_to_button(Task *t)
     {
       if ((tasks_all) || 
 	  (((Task *)(p->data))->sticky) || 
-	  (((Task *)(p->data))->desktop == current_desk))
+	  (
+	   (((Task *)(p->data))->desktop == current_desk)
+	  && (((Task *)(p->data))->ax == area_x)
+	  && (((Task *)(p->data))->ay == area_y)
+	   )
+	  )	
 	num++;
       p = p->next;
     }
@@ -2164,10 +2476,7 @@ set_task_info_to_button(Task *t)
     }
   else
     {
-      if (pager_size)
-	mw = pager_rows * PAGER_W_0;
-      else
-	mw = pager_rows * PAGER_W_1;		
+      mw = max_task_vwidth;
       if (t->name)
 	len = strlen(t->name);
       else
@@ -2221,7 +2530,12 @@ populate_tasks(void)
     {
       if ((tasks_all) || 
 	  (((Task *)(p->data))->sticky) || 
-	  (((Task *)(p->data))->desktop == current_desk))
+	  (
+	   (((Task *)(p->data))->desktop == current_desk)
+	  && (((Task *)(p->data))->ax == area_x)
+	  && (((Task *)(p->data))->ay == area_y)
+	   )
+	  )
 	num++;
       p = p->next;
     }
@@ -2253,13 +2567,18 @@ populate_tasks(void)
 	  gtk_object_set_data(GTK_OBJECT(button), "task", t);
 	}
       
-      if (((!(tasks_all)) && (t->desktop == current_desk)) || (tasks_all) ||
+      if (((!(tasks_all)) && 
+	   (t->desktop == current_desk)
+	   && (t->ax == area_x)
+	   && (t->ay == area_y)
+	   )
+	  || (tasks_all) ||
 	  (t->sticky))
 	{
 	  hbox = gtk_hbox_new(0, FALSE);
 	  gtk_widget_show(hbox);
 	  
-	  label = gtk_label_new(t->name);
+	  label = gtk_label_new(t->name?t->name:"???");
 	  gtk_widget_show(label);
 
 	  icon1 = icon2 = icon3 = NULL;
