@@ -25,6 +25,9 @@
 void 
 showHelp(AppletWidget *applet, gpointer data)
 {
-    static GnomeHelpMenuEntry help_entry = {"mini-commander", "index.html"};
+    static GnomeHelpMenuEntry help_entry = { NULL, "index.html"};
+
+    help_entry.name = gnome_app_id;
+    
     gnome_help_display(NULL, &help_entry);
 }
