@@ -1,6 +1,11 @@
-/*###################################################################*/
-/*##                       gmenu (GNOME menu editor) 0.3.1         ##*/
-/*###################################################################*/
+/*
+ * This file is a part of gmenu, the GNOME panel menu editor.
+ *
+ * File: gmenu.h
+ *
+ * This is the header file.
+ * 
+ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -15,8 +20,8 @@
 
 	/* definitions */
 #define GMENU_VERSION_MAJOR 0
-#define GMENU_VERSION_MINOR 3
-#define GMENU_VERSION_REV 2
+#define GMENU_VERSION_MINOR 99
+#define GMENU_VERSION_REV 0
 
 typedef struct _Desktop_Data Desktop_Data;
 struct _Desktop_Data
@@ -70,7 +75,9 @@ void tree_sort_node(GtkCTreeNode *node);
 GtkCTreeNode *find_file_in_tree(GtkCTree * ctree, char *path);
 void move_down_cb(GtkWidget *w, gpointer data);
 void move_up_cb(GtkWidget *w, gpointer data);
-void tree_item_selected (GtkCTree *ctree, GdkEventButton *event, gpointer data);
+void tree_item_selected (GtkCTree *ctree, GdkEventButton *event,
+			 gpointer data);
+gint tree_row_selected (GtkCTree *ctree, GtkCTreeNode *node, gint column);
 GtkCTreeNode *add_leaf_node(GtkCTree *ctree, GtkCTreeNode *parent, GtkCTreeNode *node, char *file);
 void add_tree_node(GtkCTree *ctree, GtkCTreeNode *parent, GtkWidget *pbar);
 void add_main_tree_node(void);
@@ -84,6 +91,7 @@ int is_node_moveable(GtkCTreeNode *node);
 
 
 void edit_area_select_name(void);
+void disable_edit_area(void);
 void edit_area_reset_revert(Desktop_Data *d);
 gchar * edit_area_get_filename(void);
 void update_edit_area(Desktop_Data *d);
