@@ -1,10 +1,14 @@
 /*
  * The GNOME terminal, using Michael Zucchi's zvt widget.
- * (C) 1998 The Free Software Foundation
+ * (C) 1998, 1999 The Free Software Foundation
  *
  * Authors: Miguel de Icaza (GNOME terminal)
  *          Erik Troan      (various configuration enhancements)
- *          Michael Zucchi (zvt widget, font code).
+ *          Michael Zucchi  (zvt widget, font code).
+ *
+ * Other contributors: George Lebl, Jeff Garzik, Jay Painter,
+ * Christopher Blizzard, Jens Lautenbacher, Tom Tromey, Tristan Tarant
+ * and Jonathan Blandford
  */
 #include <config.h>
 #include <unistd.h>
@@ -31,7 +35,7 @@ char **env;
 char *geometry = 0;
 
 /* By default record utmp logins */
-gboolean update_utmp = TRUE;
+gboolean update_records = ZVT_TERM_DO_UTMP_LOG | ZVT_TERM_DO_WTMP_LOG;
 
 /* is there pixmap compiled into zvt */
 static gboolean zvt_pixmap_support = FALSE;
@@ -78,6 +82,7 @@ struct terminal_config {
 	int background_pixmap;
 	char * pixmap_file;
         char * window_title;                    /* the window title */
+	char * wordclass;			/* select-by-word character class */
 } ;
 
 /* Initial command */
@@ -96,6 +101,7 @@ typedef struct {
 	GtkWidget *scroll_kbd_checkbox;
 	GtkWidget *scroll_out_checkbox;
 	GtkWidget *swapkeys_checkbox;
+	GtkWidget *wordclass_entry;
 	GtkWidget *pixmap_checkbox;
 	GtkWidget *pixmap_file_entry;
 	GtkWidget *transparent_checkbox;
@@ -126,7 +132,7 @@ about_terminal_cmd (void)
 
         const gchar *authors[] = {
 		"Zvt terminal widget: "
-		"    Michael Zucchi (zucchi@zedzone.box.net.au)",
+		"    Michael Zucchi (zucchi@zedzone.mmc.com.au)",
 		"GNOME terminal: "
 		"    Miguel de Icaza (miguel@kernel.org)",
 		"    Erik Troan (ewt@redhat.com)",
@@ -134,7 +140,7 @@ about_terminal_cmd (void)
 	};
 
         about = gnome_about_new (_("GNOME Terminal"), VERSION,
-				 "(C) 1998 the Free Software Foundation",
+				 "(C) 1998, 1999 the Free Software Foundation",
 				 authors,
 				 _("The GNOME terminal emulation program."),
 				 NULL);
@@ -340,6 +346,7 @@ load_config (char *class)
 
 	cfg->scrollback = gnome_config_get_int ("scrollbacklines=100");
 	cfg->font    = gnome_config_get_string ("font=" DEFAULT_FONT);
+	cfg->wordclass  = gnome_config_get_string ("wordclass=-A-Za-z0-9,./?%&#");
 	p = gnome_config_get_string ("scrollpos=left");
 	if (strcasecmp (p, "left") == 0)
 		cfg->scrollbar_position = SCROLLBAR_LEFT;
@@ -415,7 +422,7 @@ gather_changes (ZvtTerm *term)
 
 	if (!strcmp (gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (prefs->class_box)->entry)),
 			_("Default")))
-		newcfg->class = g_strdup (_("Config"));
+		newcfg->class = g_strdup ("Config");
 	else
 		newcfg->class = g_strconcat ("Class-", gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (prefs->class_box)->entry)), NULL);
 
@@ -443,6 +450,7 @@ terminal_config_dup (struct terminal_config *cfg)
 	*n = *cfg;
 	n->class = g_strdup (cfg->class);
 	n->font = g_strdup  (cfg->font);
+	n->wordclass = g_strdup  (cfg->wordclass);
 
 	return n;
 }
@@ -452,6 +460,7 @@ terminal_config_free (struct terminal_config *cfg)
 {
 	g_free (cfg->class);
 	g_free (cfg->font);
+	g_free (cfg->wordclass);
 	g_free (cfg);
 }
 
@@ -472,6 +481,7 @@ apply_changes (ZvtTerm *term, struct terminal_config *newcfg)
 
 	/*zvt_term_set_font_name (term, cfg->font);*/
 	gnome_term_set_font (term, cfg->font);
+	zvt_term_set_wordclass (term, cfg->wordclass);
 	zvt_term_set_bell(term, !cfg->bell);
 	zvt_term_set_blink (term, cfg->blink);
 	zvt_term_set_scroll_on_keystroke (term, cfg->scroll_key);
@@ -773,6 +783,7 @@ enum {
 	MENUBAR_ROW     = 4,
 	BELL_ROW        = 5,
 	SWAPKEYS_ROW    = 6,
+	WORDCLASS_ROW	= 7,
 	PIXMAP_ROW	= 1,
 	PIXMAP_FILE_ROW	= 2,
 	TRANSPARENT_ROW = 3,
@@ -804,6 +815,8 @@ save_preferences (GtkWidget *widget, ZvtTerm *term,
 {
 	if (cfg->font)
 		gnome_config_set_string ("font", cfg->font);
+	if (cfg->wordclass)
+		gnome_config_set_string ("wordclass", cfg->wordclass);
 	gnome_config_set_string ("scrollpos",
 				 cfg->scrollbar_position == SCROLLBAR_LEFT ? "left" :
 				 cfg->scrollbar_position == SCROLLBAR_RIGHT ? "right" : "hidden");
@@ -969,6 +982,18 @@ preferences_cmd (GtkWidget *widget, ZvtTerm *term)
 	gtk_table_attach (GTK_TABLE (table), prefs->swapkeys_checkbox,
 			  2, 3, SWAPKEYS_ROW, SWAPKEYS_ROW+1, GTK_FILL, 0, GNOME_PAD, GNOME_PAD);
 	
+	/* Word selection class */
+	l = aligned_label (_("Select-by-word characters"));
+	gtk_table_attach (GTK_TABLE (table), l,
+                          1, 2, WORDCLASS_ROW, WORDCLASS_ROW+1, GTK_FILL, 0, GNOME_PAD, GNOME_PAD);
+	prefs->wordclass_entry = gtk_entry_new();
+	gtk_entry_set_text (GTK_ENTRY (prefs->wordclass_entry),
+			    cfg->wordclass);
+	gtk_signal_connect (GTK_OBJECT (prefs->wordclass_entry), "changed",
+			    GTK_SIGNAL_FUNC (prop_changed), prefs);
+	gtk_table_attach (GTK_TABLE (table), prefs->wordclass_entry,
+			  2, 3, WORDCLASS_ROW, WORDCLASS_ROW+1, GTK_FILL, 0, GNOME_PAD, GNOME_PAD);
+	
 	/* Image page */
 	/* if pixmap support isn't in zvt, we still create the widgets for
 	   the page, so that we can query them later, but they won't be shown
@@ -1161,6 +1186,7 @@ show_menu_cmd (GtkWidget *widget, ZvtTerm *term)
 	cfg = gtk_object_get_data (GTK_OBJECT (term), "config");
 	cfg->menubar_hidden = 0;
 	gtk_widget_show (app->menubar->parent);
+	save_preferences_cmd (widget, term);
 }
 
 static void
@@ -1173,6 +1199,7 @@ hide_menu_cmd (GtkWidget *widget, ZvtTerm *term)
 	cfg = gtk_object_get_data (GTK_OBJECT (term), "config");
 	cfg->menubar_hidden = 1;
 	gtk_widget_hide (app->menubar->parent);
+	save_preferences_cmd (widget, term);
 }
 
 static GnomeUIInfo gnome_terminal_terminal_menu [] = {
@@ -1558,6 +1585,7 @@ new_terminal_cmd (char **cmd, struct terminal_config *cfg_in, gchar *geometry)
 
 	zvt_term_set_scrollback (term, cfg->scrollback);
 	gnome_term_set_font (term, cfg->font);
+	zvt_term_set_wordclass  (term, cfg->wordclass);
 	zvt_term_set_bell  (term, !cfg->bell);
 	zvt_term_set_blink (term, cfg->blink);
 	zvt_term_set_scroll_on_keystroke (term, cfg->scroll_key);
@@ -1580,8 +1608,6 @@ new_terminal_cmd (char **cmd, struct terminal_config *cfg_in, gchar *geometry)
 				  GTK_SIGNAL_FUNC (size_allocate), term);
 	
 	gnome_app_create_menus_with_data (GNOME_APP (app), gnome_terminal_menu, term);
-	if (cfg->menubar_hidden)
-		gtk_widget_hide (GNOME_APP (app)->menubar->parent);
 	
 	/* Decorations */
 	hbox = gtk_hbox_new (0, 0);
@@ -1636,12 +1662,20 @@ new_terminal_cmd (char **cmd, struct terminal_config *cfg_in, gchar *geometry)
 		zvt_term_set_background (term, NULL, 0, 0);
 
 	gtk_widget_show (app);
+
+	/*
+	 * We need to hide this here, because the gnome_app_show
+	 * method will force a show on the menu
+	 */
+	if (cfg->menubar_hidden)
+		gtk_widget_hide (GNOME_APP (app)->menubar->parent);
+	
 	set_color_scheme (term, cfg);
 
 	gdk_window_set_hints (((GtkWidget *)app)->window,
 			      0, 0, 50, 50, 0, 0, GDK_HINT_MIN_SIZE);
 
-	switch (zvt_term_forkpty (term, update_utmp)){
+	switch (zvt_term_forkpty (term, update_records)){
 	case -1:
 		perror ("Error: unable to fork");
 		return;
@@ -1692,7 +1726,7 @@ load_session ()
 	for (i = 0; i < num_terms; ++i){
 		char *geom, **argv;
 		struct terminal_config *cfg;
-		char *class;
+		char *class, *class_path;
 		int argc;
 		char *prefix = g_strdup_printf ("%s%d/", file, i);
 	  
@@ -1702,7 +1736,10 @@ load_session ()
 		   all session data on geometry! */
 		geom = gnome_config_get_string ("geometry");
 		gnome_config_get_vector ("command", &argc, &argv);
-		class = gnome_config_get_string("class=Default");
+
+		class_path = g_strconcat ("class=", _("Default"), NULL);
+		class = gnome_config_get_string(class_path);
+		g_free (class_path);
 
 		cfg = load_config (class);
 		g_free(class);
@@ -1829,7 +1866,9 @@ enum {
 	CLASS_KEY    = -8,
 	DOUTMP_KEY   = -9,
 	DONOUTMP_KEY = -10,
-        TITLE_KEY    = -11
+	DOWTMP_KEY   = -11,
+	DONOWTMP_KEY = -12,
+        TITLE_KEY    = -13
 };
 
 static struct poptOption cb_options [] = {
@@ -1860,10 +1899,16 @@ static struct poptOption cb_options [] = {
 	  N_("Background color"), N_("COLOR")},
 
 	{ "utmp", '\0', POPT_ARG_NONE, NULL, DOUTMP_KEY,
-	  N_("Update utmp/wtmp entries"), N_("UTMP") },
+	  N_("Update utmp entry"), N_("UTMP") },
 
 	{ "noutmp", '\0', POPT_ARG_NONE, NULL, DONOUTMP_KEY,
-	  N_("Do not update utmp/wtmp entries"), N_("NOUTMP") },
+	  N_("Do not update utmp entry"), N_("NOUTMP") },
+
+	{ "wtmp", '\0', POPT_ARG_NONE, NULL, DOWTMP_KEY,
+	  N_("Update wtmp entry"), N_("WTMP") },
+
+	{ "nowtmp", '\0', POPT_ARG_NONE, NULL, DONOWTMP_KEY,
+	  N_("Do not update wtmp entry"), N_("NOWTMP") },
 	
 	{ "title", 't', POPT_ARG_STRING, NULL, TITLE_KEY,
           N_("Set the window title"), N_("TITLE") },
@@ -1914,19 +1959,25 @@ parse_an_arg (poptContext state,
 		  }
 	case FORE_KEY:
 	        cfg->user_fore_str = arg;
-		//		gdk_color_parse(cfg->user_fore_str, &cfg->user_fore);
+		/* gdk_color_parse(cfg->user_fore_str, &cfg->user_fore); */
 		cfg->have_user_colors = 1;
 		break;
 	case BACK_KEY:
 	        cfg->user_back_str = arg;
-		//gdk_color_parse(cfg->user_back_str, &cfg->user_back);
+		/* gdk_color_parse(cfg->user_back_str, &cfg->user_back); */
 		cfg->have_user_colors = 1;
 		break;
 	case DOUTMP_KEY:
-		update_utmp = TRUE;
+		update_records |= ZVT_TERM_DO_UTMP_LOG;
 		break;
 	case DONOUTMP_KEY:
-		update_utmp = FALSE;
+		update_records &= ~ZVT_TERM_DO_UTMP_LOG;
+		break;
+	case DOWTMP_KEY:
+		update_records |= ZVT_TERM_DO_WTMP_LOG;
+		break;
+	case DONOWTMP_KEY:
+		update_records &= ~ZVT_TERM_DO_WTMP_LOG;
 		break;
 	case TITLE_KEY:
 	        cfg->window_title = g_strdup(arg);
@@ -1949,7 +2000,6 @@ main_terminal_program (int argc, char *argv [], char **environ)
 	char *program_name;
 	char *class;
 	struct terminal_config *default_config, *cmdline_config;
-	poptContext ctx;
 
 	env = environ;
 	
@@ -1961,7 +2011,7 @@ main_terminal_program (int argc, char *argv [], char **environ)
 	cb_options[0].descrip = (char *)cmdline_config;
 	
 	gnome_init_with_popt_table("Terminal", VERSION, argc, argv,
-				   cb_options, 0, &ctx);
+				   cb_options, 0, NULL);
 
 #if 1
 	if(cmdline_config->user_back_str)
