@@ -62,9 +62,12 @@ accept_connection (gpointer client_data, gint source,
       gtk_main_iteration ();
       status2 = IceConnectionStatus (connection);
     }
-
+#if 0
+  /* I think that the changes to gnome-ice.c mean that the connection
+     is being closed before we get here - FIXME: check this */
   if (status2 != IceConnectAccepted)
     IceCloseConnection (connection);
+#endif
 }
 
 /* FIXME: error message text could be returned somehow.  Or at least
@@ -100,10 +103,8 @@ initialize_ice (void)
 
   ids = IceComposeNetworkIdList (num_transports, listeners);
 #define ENVNAME "SESSION_MANAGER"
-  p = g_new (char, sizeof ENVNAME + strlen (ids) + 1);
-  sprintf (p, "%s=%s", ENVNAME, ids);
+  p = g_strconcat (ENVNAME "=", ids, NULL);
   putenv (p);
-  g_free(p);
 
   return 1;
 }
