@@ -23,7 +23,7 @@ DOCUMENTATION.
 **
 **	(c) COPYRIGHT MIT 1995.
 **	Please first read the full copyright statement in the file COPYRIGH.
-**	@(#) $Id: HTParse.c,v 1.3 1998/05/15 17:54:10 sopwith Exp $
+**	@(#) $Id: HTParse.c,v 1.4 1998/07/07 06:23:22 tromey Exp $
 **
 ** history:
 **	May 12 94	TAB added as legal char in HTCleanTelnetString
@@ -33,9 +33,6 @@ DOCUMENTATION.
 #define GHELP
 #if defined(GHELP)
 #include <string.h>
-#if !defined(__FreeBSD__)
-#include <malloc.h>
-#endif
 #include <stdlib.h>
 #include <stdio.h>
 #include <ctype.h>
