@@ -99,14 +99,14 @@ corner_widget_realize(GtkWidget *w)
 
 	gnome_win_hints_init();
 	if (gnome_win_hints_wm_exists()) {
-		gnome_win_hints_set_hints(w,
-					  WIN_HINTS_SKIP_FOCUS |
-					  WIN_HINTS_SKIP_WINLIST |
-					  WIN_HINTS_SKIP_TASKBAR);
+		gnome_win_hints_set_hints(w, GNOME_PANEL_HINTS |
+					  WIN_HINTS_DO_NOT_COVER);
 		gnome_win_hints_set_state(w,
 					  WIN_STATE_STICKY |
 					  WIN_STATE_FIXED_POSITION);
-		gnome_win_hints_set_layer(w, WIN_LAYER_DOCK);
+		gnome_win_hints_set_layer(w, global_config.keep_bottom?
+					  WIN_LAYER_BELOW:
+					  WIN_LAYER_DOCK);
 		gnome_win_hints_set_expanded_size(w, 0, 0, 0, 0);
 		gdk_window_set_decorations(w->window, 0);
 	}
@@ -178,12 +178,11 @@ corner_widget_size_request(GtkWidget *widget,
 
 static void
 corner_widget_get_hidepos(CornerWidget *corner, PanelOrientType *hide_orient,
-			  int *w, int *h)
+			  gint16 *w, gint16 *h)
 {
 	BasePWidget *basep = BASEP_WIDGET(corner);
 	PanelWidget *panel = PANEL_WIDGET(basep->panel);
-	*w = GTK_WIDGET(corner)->allocation.width;
-	*h = GTK_WIDGET(corner)->allocation.height;
+
 	*hide_orient = -1;
 	if(corner->state == CORNER_SHOWN)
 		return;
@@ -296,18 +295,13 @@ corner_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 
 	if(corner->state != CORNER_SHOWN) {
 		PanelOrientType hide_orient;
-		int x,y;
-		int w = allocation->width;
-		int h = allocation->height;
 		corner_widget_get_hidepos(corner, &hide_orient,
-					  &w, &h);
+					  &allocation->width,
+					  &allocation->height);
 		basep_widget_get_position(basep, hide_orient,
-					  &x, &y, w, h);
-		challoc.x = x;
-		challoc.y = y;
-
-		allocation->width = w;
-		allocation->height = h;
+					  &challoc.x, &challoc.y,
+					  allocation->width,
+					  allocation->height);
 	}
 
 	widget->allocation = *allocation;
@@ -370,7 +364,12 @@ corner_widget_pop_show(CornerWidget *corner, int fromright)
 						pw_explicit_step);
 	}
 
+	gnome_win_hints_set_hints(GTK_WIDGET(corner),
+				  GNOME_PANEL_HINTS |
+				  WIN_HINTS_DO_NOT_COVER);
 	gnome_win_hints_set_layer(GTK_WIDGET(corner),
+				  global_config.keep_bottom?
+				  WIN_LAYER_BELOW:
 				  WIN_LAYER_DOCK);
 
 	corner->state = CORNER_SHOWN;
@@ -391,7 +390,11 @@ corner_widget_pop_hide(CornerWidget *corner, int fromright)
 
 	gnome_triggers_vdo("", NULL, supinfo);
 	
+	gnome_win_hints_set_hints(GTK_WIDGET(corner),
+				  GNOME_PANEL_HINTS);
 	gnome_win_hints_set_layer(GTK_WIDGET(corner),
+				  global_config.keep_bottom?
+				  WIN_LAYER_ONTOP:
 				  WIN_LAYER_ABOVE_DOCK);
 
 	gtk_signal_emit(GTK_OBJECT(corner),
