@@ -99,14 +99,13 @@ drawer_widget_realize(GtkWidget *w)
 
 	gnome_win_hints_init();
 	if (gnome_win_hints_wm_exists()) {
-		gnome_win_hints_set_hints(w,
-					  WIN_HINTS_SKIP_FOCUS |
-					  WIN_HINTS_SKIP_WINLIST |
-					  WIN_HINTS_SKIP_TASKBAR);
+		gnome_win_hints_set_hints(w, GNOME_PANEL_HINTS);
 		gnome_win_hints_set_state(w,
 					  WIN_STATE_STICKY |
 					  WIN_STATE_FIXED_POSITION);
-		gnome_win_hints_set_layer(w, WIN_LAYER_ABOVE_DOCK);
+		gnome_win_hints_set_layer(w, global_config.keep_bottom?
+					  WIN_LAYER_BELOW:
+					  WIN_LAYER_ABOVE_DOCK);
 		gnome_win_hints_set_expanded_size(w, 0, 0, 0, 0);
 		gdk_window_set_decorations(w->window, 0);
 	}
@@ -294,10 +293,7 @@ drawer_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 				      allocation->x,
 				      allocation->y,
 				      0,0,0,0, GDK_HINT_POS);
-	} else if(widget->window) {
-		puts("NON_REALIZED_WINDOW");
 	}
-	
 
 	challoc.x = challoc.y = 0;
 	challoc.width = allocation->width;
