@@ -424,53 +424,52 @@ gtk_style_shade (GdkColor *a,
 #define LIGHTNESS_MULT  1.3
 #define DARKNESS_MULT   0.7
 
+static void
+set_color_back(GtkWidget *widget, PanelWidget *panel)
+{
+	GtkStyle *ns;
+	int i;
+
+	ns = gtk_style_copy(widget->style);
+	gtk_style_ref(ns);
+
+	ns->bg[GTK_STATE_NORMAL] =
+		panel->back_color;
+	gtk_style_shade (&panel->back_color,
+			 &ns->bg[GTK_STATE_PRELIGHT],1.5);
+	gtk_style_shade (&panel->back_color,
+			 &ns->bg[GTK_STATE_ACTIVE],0.8);
+	ns->bg[GTK_STATE_INSENSITIVE] = 
+		panel->back_color;
+
+	for (i = 0; i < 5; i++) {
+		gtk_style_shade (&ns->bg[i], &ns->light[i], LIGHTNESS_MULT);
+		gtk_style_shade (&ns->bg[i], &ns->dark[i], DARKNESS_MULT);
+
+		ns->mid[i].red = (ns->light[i].red + ns->dark[i].red) / 2;
+		ns->mid[i].green = (ns->light[i].green + ns->dark[i].green) / 2;
+		ns->mid[i].blue = (ns->light[i].blue + ns->dark[i].blue) / 2;
+	}
+	gtk_widget_set_style(widget, ns);
+	gtk_style_unref(ns);
+}
+
 void
 set_frame_colors(PanelWidget *panel, GtkWidget *frame,
 		 GtkWidget *but1, GtkWidget *but2, GtkWidget *but3, GtkWidget *but4)
 {
 	if(panel->back_type == PANEL_BACK_COLOR) {
-		GtkStyle *ns;
-		int i;
-
-		ns = gtk_style_copy(GTK_WIDGET(panel)->style);
-		gtk_style_ref(ns);
-
-		ns->bg[GTK_STATE_NORMAL] =
-			panel->back_color;
-		gtk_style_shade (&panel->back_color,
-				 &ns->bg[GTK_STATE_PRELIGHT],1.5);
-		gtk_style_shade (&panel->back_color,
-				 &ns->bg[GTK_STATE_ACTIVE],0.8);
-		ns->bg[GTK_STATE_INSENSITIVE] = 
-			panel->back_color;
-
-		for (i = 0; i < 5; i++) {
-			gtk_style_shade (&ns->bg[i], &ns->light[i], LIGHTNESS_MULT);
-			gtk_style_shade (&ns->bg[i], &ns->dark[i], DARKNESS_MULT);
-
-			ns->mid[i].red = (ns->light[i].red + ns->dark[i].red) / 2;
-			ns->mid[i].green = (ns->light[i].green + ns->dark[i].green) / 2;
-			ns->mid[i].blue = (ns->light[i].blue + ns->dark[i].blue) / 2;
-		}
-		gtk_widget_set_style(frame, ns);
-		gtk_widget_set_style(but1, ns);
-		gtk_widget_set_style(but2, ns);
-		gtk_widget_set_style(but3, ns);
-		gtk_widget_set_style(but4, ns);
-		gtk_style_unref(ns);
+		set_color_back(frame,panel);
+		set_color_back(but1,panel);
+		set_color_back(but2,panel);
+		set_color_back(but3,panel);
+		set_color_back(but4,panel);
 	} else {
-		GtkStyle *ns;
-
-		ns = gtk_rc_get_style(frame);
-		if(!ns) ns = gtk_style_new();
-
-		gtk_style_ref(ns);
-		gtk_widget_set_style(frame, ns);
-		gtk_widget_set_style(but1, ns);
-		gtk_widget_set_style(but2, ns);
-		gtk_widget_set_style(but3, ns);
-		gtk_widget_set_style(but4, ns);
-		gtk_style_unref(ns);
+		gtk_widget_restore_default_style(frame);
+		gtk_widget_restore_default_style(but1);
+		gtk_widget_restore_default_style(but2);
+		gtk_widget_restore_default_style(but3);
+		gtk_widget_restore_default_style(but4);
 	}
 }
 
