@@ -54,6 +54,9 @@ config_apply (GtkWidget *widget, int page, gpointer data)
 {
 	int i;
 	
+	if(page!=-1)
+		return;
+	
 	for(i=0;i<LAST_TILE;i++) {
 		g_free(global_config.tile_up[i]);
 		g_free(global_config.tile_down[i]);
@@ -522,7 +525,7 @@ misc_notebook_page(void)
 			    CONFIG_PADDING_SIZE);
 
 	/* Autoraise */
-	button = gtk_check_button_new_with_label (_("Raise panels on mouse-over"));
+	button = gtk_check_button_new_with_label (_("Raise panels on mouse-over (non GNOME compliant window managers only)"));
 	if (temp_config.autoraise)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
 	gtk_signal_connect (GTK_OBJECT (button), "toggled", 
@@ -531,6 +534,26 @@ misc_notebook_page(void)
 	gtk_box_pack_start (GTK_BOX (box), button, FALSE, FALSE,
 			    CONFIG_PADDING_SIZE);
 
+	/* Keep on bottom */
+	button = gtk_check_button_new_with_label (_("Keep panel below windows (GNOME compliant window managers only)"));
+	if (temp_config.keep_bottom)
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
+	gtk_signal_connect (GTK_OBJECT (button), "toggled", 
+			    GTK_SIGNAL_FUNC (set_toggle_button_value), 
+			    &(temp_config.keep_bottom));
+	gtk_box_pack_start (GTK_BOX (box), button, FALSE, FALSE,
+			    CONFIG_PADDING_SIZE);
+
+	/* Drawer/launcher auto close */
+	button = gtk_check_button_new_with_label (_("Close drawer if a launcher inside it is pressed"));
+	if (temp_config.drawer_auto_close)
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
+	gtk_signal_connect (GTK_OBJECT (button), "toggled", 
+			    GTK_SIGNAL_FUNC (set_toggle_button_value), 
+			    &(temp_config.drawer_auto_close));
+	gtk_box_pack_start (GTK_BOX (box), button, FALSE, FALSE,
+			    CONFIG_PADDING_SIZE);
+
 	/* Minimize Delay scale frame */
 	frame = make_int_scale_frame(_("Applet padding"),
 				      &(temp_config.applet_padding),
@@ -549,10 +572,10 @@ panel_config_global(void)
 	GtkWidget *box;
 	GtkWidget *prop_nbook;
 	char *icon_titles[]={
-		_("Launcher icon"),
-		_("Drawer icon"),
-		_("Menu icon"),
-		_("Logout icon")};
+		N_("Launcher icon"),
+		N_("Drawer icon"),
+		N_("Menu icon"),
+		N_("Logout icon")};
 	int i;
 	
 	/* return if the window is already up. */
