@@ -21,6 +21,8 @@ struct _GlobalConfig {
 	int disable_animations;
 	int applet_padding;
 	int autoraise;
+	int keep_bottom;
+	int drawer_auto_close;
 	int tiles_enabled[LAST_TILE];
 	char *tile_up[LAST_TILE];
 	char *tile_down[LAST_TILE];
