@@ -73,6 +73,7 @@ apply_global_config(void)
 	static int dot_buttons_old = 0; /*doesn't matter first time this is
 					  done there are no menu applets*/
 	static int small_icons_old = 0; /*same here*/
+	static int keep_bottom_old = -1;
 	panel_widget_change_global(global_config.explicit_hide_step_size,
 				   global_config.auto_hide_step_size,
 				   global_config.drawer_step_size,
@@ -120,6 +121,39 @@ apply_global_config(void)
 	small_icons_old = global_config.show_small_icons;
 	send_tooltips_state(global_config.tooltips_enabled);
 
+	if(keep_bottom_old == -1 ||
+	   keep_bottom_old != global_config.keep_bottom) {
+		GSList *li;
+		for(li = panel_list; li != NULL; li = g_slist_next(li)) {
+			PanelData *pd = li->data;
+			if(!GTK_WIDGET_REALIZED(pd->panel))
+				continue;
+			if((IS_SNAPPED_WIDGET(pd->panel) &&
+			    SNAPPED_WIDGET(pd->panel)->mode != SNAPPED_AUTO_HIDE &&
+			    SNAPPED_WIDGET(pd->panel)->state == SNAPPED_SHOWN) ||
+			   (IS_CORNER_WIDGET(pd->panel) &&
+			    CORNER_WIDGET(pd->panel)->state == SNAPPED_SHOWN)) {
+				if(global_config.keep_bottom)
+					gnome_win_hints_set_layer(pd->panel, WIN_LAYER_BELOW);
+				else
+					gnome_win_hints_set_layer(pd->panel, WIN_LAYER_DOCK);
+			} else if(IS_DRAWER_WIDGET(pd->panel)) {
+				if(global_config.keep_bottom)
+					gnome_win_hints_set_layer(pd->panel, WIN_LAYER_BELOW);
+				else
+					gnome_win_hints_set_layer(pd->panel,
+								  WIN_LAYER_ABOVE_DOCK);
+			} else {
+				if(global_config.keep_bottom)
+					gnome_win_hints_set_layer(pd->panel, WIN_LAYER_ONTOP);
+				else
+					gnome_win_hints_set_layer(pd->panel,
+								  WIN_LAYER_ABOVE_DOCK);
+			}
+		}
+	}
+	keep_bottom_old = global_config.keep_bottom;
+
 	for(i=0;i<LAST_TILE;i++) {
 		button_widget_set_flags(i, global_config.tiles_enabled[i],
 					1,0);
@@ -130,54 +164,6 @@ apply_global_config(void)
 	}
 }
 
-/*shouldn't this be in gnome-dentry?? :)*/
-static void
-gnome_desktop_entry_save_no_sync (GnomeDesktopEntry *dentry)
-{
-	char *prefix;
-	
-	g_assert (dentry != NULL);
-	g_assert (dentry->location != NULL);
-
-	prefix = g_strconcat ("=", dentry->location, "=/Desktop Entry", NULL);
-
-	gnome_config_clean_section (prefix);
-
-	prefix = g_strconcat (prefix, "/", NULL);
-	gnome_config_push_prefix (prefix);
-	g_free (prefix);
-
-	if (dentry->name)
-		gnome_config_set_translated_string ("Name", dentry->name);
-
-	if (dentry->comment)
-		gnome_config_set_translated_string ("Comment", dentry->comment);
-
-	if (dentry->exec)
-		gnome_config_set_vector ("Exec", dentry->exec_length,
-					 (const char * const *) dentry->exec);
-
-	if (dentry->tryexec)
-		gnome_config_set_string ("TryExec", dentry->tryexec);
-
-	if (dentry->icon)
-		gnome_config_set_string ("Icon", dentry->icon);
-
-	if (dentry->geometry)
-		gnome_config_set_string ("Geometry", dentry->geometry);
-	
-	if (dentry->docpath)
-		gnome_config_set_string ("DocPath", dentry->docpath);
-
-	gnome_config_set_bool ("Terminal", dentry->terminal);
-	gnome_config_set_bool ("MultipleArgs", dentry->multiple_args);
-	
-	if (dentry->type)
-		gnome_config_set_string ("Type", dentry->type);
-
-	gnome_config_pop_prefix ();
-}
-
 static int
 send_applet_session_save (AppletInfo *info,
 			  CORBA_Object obj,
@@ -325,7 +311,7 @@ save_applet_configuration(AppletInfo *info)
 					 info->applet_id+1);
 			g_free(launcher->dentry->location);
 			launcher->dentry->location = g_strdup(buf->str);
-			gnome_desktop_entry_save_no_sync(launcher->dentry);
+			gnome_desktop_entry_save(launcher->dentry);
 
 			gnome_config_set_string("id", LAUNCHER_ID);
 			gnome_config_set_string("parameters", buf->str);
@@ -425,26 +411,22 @@ do_session_save(GnomeClient *client,
 	char *s;
 	char *session_id;
 	int i;
+	gchar *new_args[] = { "rm", "-r", NULL };
 
-	session_id = gnome_client_get_id (client);
-	if(session_id) {
-		char *new_args[3];
-
+	if (panel_cfg_path)
 		g_free(panel_cfg_path);
-		panel_cfg_path = g_strconcat("/panel.d/Session-",session_id,
-					     "/",NULL);
 
-		new_args[0] = (char *) gtk_object_get_data(GTK_OBJECT(client),
-							   "argv0");
-		new_args[1] = "--discard-session";
-		new_args[2] = session_id;
-		gnome_client_set_discard_command (client, 3, new_args);
-	}
+	if (gnome_client_get_flags(client) & GNOME_CLIENT_IS_CONNECTED &&
+	    GNOME_CLIENT (client)->restart_style != GNOME_RESTART_NEVER)
+		panel_cfg_path = g_strdup (gnome_client_get_config_prefix (client));
+	else
+		panel_cfg_path = g_strdup ("/panel.d/default/");
+
+	new_args[2] = gnome_config_get_real_path (panel_cfg_path);
+	gnome_client_set_discard_command (client, 3, new_args);
 	
 	/*DEBUG*/printf("Saving to [%s]\n",panel_cfg_path);
 
-	gnome_config_clean_file(panel_cfg_path);
-
 	/*DEBUG*/printf("Saving session: 1"); fflush(stdout);
 	if(complete_sync) {
 		GSList *li;
@@ -504,6 +486,10 @@ do_session_save(GnomeClient *client,
 				     global_config.applet_padding);
 		gnome_config_set_bool("autoraise",
 				      global_config.autoraise);
+		gnome_config_set_bool("keep_bottom",
+				      global_config.keep_bottom);
+		gnome_config_set_bool("drawer_auto_close",
+				      global_config.drawer_auto_close);
 		buf = g_string_new(NULL);
 		for(i=0;i<LAST_TILE;i++) {
 			g_string_sprintf(buf,"tiles_enabled_%d",i);
@@ -541,13 +527,26 @@ panel_config_sync(void)
 		GSList *_applets_to_sync = applets_to_sync;
 
 		applets_to_sync = NULL;
-		do_session_save(client,need_complete_save,
-				_applets_to_sync,panels_to_sync,globals_to_sync);
-		need_complete_save = FALSE;
+		if (!(gnome_client_get_flags(client) & 
+		      GNOME_CLIENT_IS_CONNECTED)) {
+			do_session_save(client,need_complete_save,
+					_applets_to_sync,panels_to_sync,
+					globals_to_sync);
+			need_complete_save = FALSE;
+			panels_to_sync = FALSE;
+			globals_to_sync = FALSE;
+			g_slist_free(_applets_to_sync);
+		} else {
+			/*prevent possible races by doing this before requesting
+			  save*/
+			need_complete_save = FALSE;
+			panels_to_sync = FALSE;
+			globals_to_sync = FALSE;
+			g_slist_free(_applets_to_sync);
+			gnome_client_request_save (client, GNOME_SAVE_LOCAL, FALSE,
+						   GNOME_INTERACT_NONE, FALSE, FALSE);
+		}
 
-		g_slist_free(_applets_to_sync);
-		panels_to_sync = FALSE;
-		globals_to_sync = FALSE;
 	}
 }
 
@@ -1059,6 +1058,10 @@ load_up_globals(void)
 
 	global_config.autoraise = gnome_config_get_bool("autoraise=TRUE");
 
+	global_config.keep_bottom = gnome_config_get_bool("keep_bottom=FALSE");
+
+	global_config.drawer_auto_close = gnome_config_get_bool("drawer_auto_close=FALSE");
+
 	for(i=0;i<LAST_TILE;i++) {
 		g_string_sprintf(buf,"tiles_enabled_%d=TRUE",i);
 		global_config.tiles_enabled[i] =
