@@ -102,14 +102,22 @@ snapped_widget_realize(GtkWidget *w)
 		gnome_win_hints_set_hints(w, 
 					  WIN_HINTS_SKIP_FOCUS |
 					  WIN_HINTS_SKIP_WINLIST |
-					  WIN_HINTS_SKIP_TASKBAR);
+					  WIN_HINTS_SKIP_TASKBAR |
+					  WIN_HINTS_DO_NOT_COVER);
 		gnome_win_hints_set_state(w, 
 					  WIN_STATE_STICKY |
 					  WIN_STATE_FIXED_POSITION);
 		if(snapped->mode == SNAPPED_AUTO_HIDE) {
-			gnome_win_hints_set_layer(w, WIN_LAYER_ABOVE_DOCK);
+			gnome_win_hints_set_hints(w, GNOME_PANEL_HINTS);
+			gnome_win_hints_set_layer(w, global_config.keep_bottom?
+						  WIN_LAYER_ONTOP:
+						  WIN_LAYER_ABOVE_DOCK);
 		} else {
-			gnome_win_hints_set_layer(w, WIN_LAYER_DOCK);
+			gnome_win_hints_set_hints(w, GNOME_PANEL_HINTS |
+						  WIN_HINTS_DO_NOT_COVER);
+			gnome_win_hints_set_layer(w, global_config.keep_bottom?
+						  WIN_LAYER_BELOW:
+						  WIN_LAYER_DOCK);
 		}
 		gnome_win_hints_set_expanded_size(w, 0, 0, 0, 0);
 		gdk_window_set_decorations(w->window, 0);
@@ -193,13 +201,11 @@ snapped_widget_size_request(GtkWidget *widget,
 
 static void
 snapped_widget_get_hidepos(SnappedWidget *snapped,
-			   PanelOrientType *hide_orient, int *w, int *h)
+			   PanelOrientType *hide_orient, gint16 *w, gint16 *h)
 {
 	BasePWidget *basep = BASEP_WIDGET(snapped);
 	PanelWidget *panel = PANEL_WIDGET(basep->panel);
-	
-	*w = GTK_WIDGET(snapped)->allocation.width;
-	*h = GTK_WIDGET(snapped)->allocation.height;
+
 	*hide_orient = -1;
 	if(snapped->state == SNAPPED_SHOWN)
 		return;
@@ -286,7 +292,7 @@ snapped_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 	  because we don't want to change our size, ever*/
 	gtk_widget_size_request (basep->ebox,
 				 &basep->ebox->requisition);
-	
+
 	/*ignore the allocation we get, we want to be this large*/
 	switch(snapped->pos) {
 		case SNAPPED_BOTTOM:
@@ -300,7 +306,7 @@ snapped_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 			allocation->width = basep->ebox->requisition.width;
 			break;
 	}
-	
+
 	snapped_widget_get_pos(snapped,
 			       &allocation->x,
 			       &allocation->y,
@@ -315,18 +321,13 @@ snapped_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 
 	if(snapped->state != SNAPPED_SHOWN) {
 		PanelOrientType hide_orient;
-		int x,y;
-		int w = allocation->width;
-		int h = allocation->height;
 		snapped_widget_get_hidepos(snapped, &hide_orient,
-					   &w, &h);
-		basep_widget_get_position(basep, hide_orient, &x, &y,
-					  w, h);
-		challoc.x = x;
-		challoc.y = y;
-
-		allocation->width = w;
-		allocation->height = h;
+					   &allocation->width,
+					   &allocation->height);
+		basep_widget_get_position(basep, hide_orient,
+					  &challoc.x, &challoc.y,
+					  allocation->width,
+					  allocation->height);
 	}
 
 	widget->allocation = *allocation;
@@ -373,7 +374,8 @@ snapped_widget_set_initial_pos(SnappedWidget *snapped)
 void
 snapped_widget_pop_up(SnappedWidget *snapped)
 {
-	if ((snapped->state == SNAPPED_MOVING) ||
+	if ((snapped->mode != SNAPPED_AUTO_HIDE) ||
+	    (snapped->state == SNAPPED_MOVING) ||
 	    (snapped->state == SNAPPED_SHOWN))
 		return;
 
@@ -428,8 +430,6 @@ snapped_widget_pop_down(gpointer data)
 	if(snapped->autohide_inhibit)
 		return TRUE;
 
-	gnome_triggers_vdo("", NULL, supinfo);
-
 	if(snapped->state != SNAPPED_SHOWN ||
 	   panel_widget_is_cursor(PANEL_WIDGET(BASEP_WIDGET(snapped)->panel),0)) {
 		snapped->leave_notify_timer_tag = 0;
@@ -442,6 +442,8 @@ snapped_widget_pop_down(gpointer data)
 	   snapped->drawers_open>0)
 		return TRUE;
 
+	gnome_triggers_vdo("", NULL, supinfo);
+
 	gtk_signal_emit(GTK_OBJECT(snapped),
 			snapped_widget_signals[STATE_CHANGE_SIGNAL],
 			SNAPPED_HIDDEN);
@@ -526,9 +528,20 @@ snapped_widget_pop_show(SnappedWidget *snapped, int fromright)
 	}
 
 	if(snapped->mode == SNAPPED_AUTO_HIDE) {
-		gnome_win_hints_set_layer(GTK_WIDGET(snapped), WIN_LAYER_ABOVE_DOCK);
+		gnome_win_hints_set_hints(GTK_WIDGET(snapped),
+					  GNOME_PANEL_HINTS);
+		gnome_win_hints_set_layer(GTK_WIDGET(snapped),
+					  global_config.keep_bottom?
+					  WIN_LAYER_ONTOP:
+					  WIN_LAYER_ABOVE_DOCK);
 	} else {
-		gnome_win_hints_set_layer(GTK_WIDGET(snapped), WIN_LAYER_DOCK);
+		gnome_win_hints_set_hints(GTK_WIDGET(snapped),
+					  GNOME_PANEL_HINTS |
+					  WIN_HINTS_DO_NOT_COVER);
+		gnome_win_hints_set_layer(GTK_WIDGET(snapped),
+					  global_config.keep_bottom?
+					  WIN_LAYER_BELOW:
+					  WIN_LAYER_DOCK);
 	}
 
 	snapped->state = SNAPPED_SHOWN;
@@ -553,7 +566,12 @@ snapped_widget_pop_hide(SnappedWidget *snapped, int fromright)
 		snapped->leave_notify_timer_tag = 0;
 	}
 
-	gnome_win_hints_set_layer(GTK_WIDGET(snapped), WIN_LAYER_ABOVE_DOCK);
+	gnome_win_hints_set_hints(GTK_WIDGET(snapped),
+				  GNOME_PANEL_HINTS);
+	gnome_win_hints_set_layer(GTK_WIDGET(snapped),
+				  global_config.keep_bottom?
+				  WIN_LAYER_ONTOP:
+				  WIN_LAYER_ABOVE_DOCK);
 
 	if(fromright)
 	   	gtk_signal_emit(GTK_OBJECT(snapped),
@@ -791,7 +809,7 @@ snapped_widget_new (SnappedPos pos,
 		snapped->state = state;
 
 	snapped->autohide_inhibit = FALSE;
-	
+
 	/*sanity check ... this is a case which should never happen*/
 	if(snapped->mode == SNAPPED_EXPLICIT_HIDE &&
 	   snapped->state == SNAPPED_HIDDEN)
@@ -834,10 +852,19 @@ snapped_widget_change_params(SnappedWidget *snapped,
 		if (gnome_win_hints_wm_exists()) {
 			GtkWidget *wid = GTK_WIDGET(snapped);
 			if(snapped->mode == SNAPPED_AUTO_HIDE) {
+				gnome_win_hints_set_hints(GTK_WIDGET(snapped),
+							  GNOME_PANEL_HINTS);
 				gnome_win_hints_set_layer(wid,
+							  global_config.keep_bottom?
+							  WIN_LAYER_ONTOP:
 							  WIN_LAYER_ABOVE_DOCK);
 			} else {
+				gnome_win_hints_set_hints(GTK_WIDGET(snapped),
+							  GNOME_PANEL_HINTS |
+							  WIN_HINTS_DO_NOT_COVER);
 				gnome_win_hints_set_layer(wid,
+							  global_config.keep_bottom?
+							  WIN_LAYER_BELOW:
 							  WIN_LAYER_DOCK);
 			}
 		}
